@@ -124,14 +124,10 @@ public:
     void
     handle_alert(lt::alert* a) override
     {
-        if (lt::alert_cast<lt::torrent_alert>(a)) {
-            auto* x = lt::alert_cast<lt::torrent_alert>(a);
+        if (auto* x = lt::alert_cast<lt::read_piece_alert>(a)) {
             if (x->handle.info_hash() != m_ih)
                 return;
-        }
 
-        if (lt::alert_cast<lt::read_piece_alert>(a)) {
-            auto* x = lt::alert_cast<lt::read_piece_alert>(a);
             if (x->piece != m_piece)
                 return;
 
@@ -161,14 +157,10 @@ public:
     void
     handle_alert(lt::alert* a) override
     {
-        if (lt::alert_cast<lt::torrent_alert>(a)) {
-            auto* x = lt::alert_cast<lt::torrent_alert>(a);
+        if (auto* x = lt::alert_cast<lt::piece_finished_alert>(a)) {
             if (x->handle.info_hash() != m_ih)
                 return;
-        }
 
-        if (lt::alert_cast<lt::piece_finished_alert>(a)) {
-            auto* x = lt::alert_cast<lt::piece_finished_alert>(a);
             if (x->piece_index != m_piece)
                 return;
 
@@ -194,19 +186,22 @@ public:
     void
     handle_alert(lt::alert* a) override
     {
-        if (lt::alert_cast<lt::torrent_alert>(a)) {
-            auto* x = lt::alert_cast<lt::torrent_alert>(a);
+        if (auto* x = lt::alert_cast<lt::torrent_error_alert>(a)) {
             if (x->handle.info_hash() != m_ih)
                 return;
-        }
 
-        if (lt::alert_cast<lt::torrent_error_alert>(a)) {
             set_exception(
                 std::make_exception_ptr(std::runtime_error("metadata failed")));
-        } else if (lt::alert_cast<lt::metadata_failed_alert>(a)) {
+        } else if (auto* x = lt::alert_cast<lt::metadata_failed_alert>(a)) {
+            if (x->handle.info_hash() != m_ih)
+                return;
+
             set_exception(
                 std::make_exception_ptr(std::runtime_error("metadata failed")));
-        } else if (lt::alert_cast<lt::metadata_received_alert>(a)) {
+        } else if (auto* x = lt::alert_cast<lt::metadata_received_alert>(a)) {
+            if (x->handle.info_hash() != m_ih)
+                return;
+
             // Metadata download is done
             set_value();
         }
@@ -226,8 +221,7 @@ public:
     void
     handle_alert(lt::alert* a) override
     {
-        if (lt::alert_cast<lt::torrent_removed_alert>(a)) {
-            auto* x = lt::alert_cast<lt::torrent_removed_alert>(a);
+        if (auto* x = lt::alert_cast<lt::torrent_removed_alert>(a)) {
             if (x->info_hash != m_ih)
                 return;
 
@@ -254,8 +248,6 @@ Download::Download(std::mutex& mtx, lt::add_torrent_params& atp, bool k)
 
     // Need to give libtorrent some time to breethe
     std::this_thread::sleep_for(std::chrono::milliseconds(500));
-
-    download_metadata();
 }
 
 Download::~Download()
@@ -276,11 +268,14 @@ Download::~Download()
 }
 
 ssize_t
-Download::read(int file, int64_t fileoff, char* buf, size_t buflen)
+Download::read(int file, int64_t fileoff, char* buf, size_t buflen,
+    DataProgressCb progress_cb)
 {
     D(printf("%s:%d: %s(%d, %lu, %p, %lu)\n", __FILE__, __LINE__, __func__,
         file, fileoff, buf, buflen));
 
+    download_metadata();
+
     auto ti = m_th.torrent_file();
 
     auto fs = ti->files();
@@ -319,7 +314,7 @@ Download::read(int file, int64_t fileoff, char* buf, size_t buflen)
     set_piece_priority(file, fileoff, (int) p5, PRIO_HIGH);
 
     if (!m_th.have_piece(part.piece))
-        download(part);
+        download(part, progress_cb);
 
     return read(part, buf, buflen);
 }
@@ -327,6 +322,10 @@ Download::read(int file, int64_t fileoff, char* buf, size_t buflen)
 void
 Download::set_piece_priority(int file, int64_t off, int size, int prio)
 {
+    D(printf("%s:%d: %s()\n", __FILE__, __LINE__, __func__));
+
+    download_metadata();
+
     auto ti = m_th.torrent_file();
 
     auto fs = ti->files();
@@ -348,6 +347,10 @@ Download::set_piece_priority(int file, int64_t off, int size, int prio)
 std::vector<std::pair<std::string, uint64_t>>
 Download::get_files()
 {
+    D(printf("%s:%d: %s()\n", __FILE__, __LINE__, __func__));
+
+    download_metadata();
+
     std::vector<std::pair<std::string, uint64_t>> files;
 
     const lt::file_storage& fs = m_th.torrent_file()->files();
@@ -358,9 +361,12 @@ Download::get_files()
     return files;
 }
 
+// static
 std::vector<std::pair<std::string, uint64_t>>
 Download::get_files(char* metadata, size_t metadatasz)
 {
+    D(printf("%s:%d: %s()\n", __FILE__, __LINE__, __func__));
+
     lt::error_code ec;
 
     lt::torrent_info ti(metadata, (int) metadatasz, ec);
@@ -377,10 +383,13 @@ Download::get_files(char* metadata, size_t metadatasz)
     return files;
 }
 
+// static
 std::shared_ptr<std::vector<char>>
-Download::get_metadata(
-    std::string url, std::string save_path, std::string cache_path)
+Download::get_metadata(std::string url, std::string save_path,
+    std::string cache_path, MetadataProgressCb cb)
 {
+    D(printf("%s:%d: %s()\n", __FILE__, __LINE__, __func__));
+
     lt::add_torrent_params atp;
     atp.save_path = save_path;
     atp.flags &= ~lt::add_torrent_params::flag_auto_managed;
@@ -411,22 +420,34 @@ Download::get_metadata(
             // The torrent_info from was is invalid
             atp.ti = NULL;
 
-            // Download metadata
-            auto dl = Download::get_download(atp, true /* keep */);
+            // Dowload metadata
+            auto metadata = Download::get_download(atp, true)->get_metadata(cb);
 
-            return dl->get_metadata_and_write_to_file(path);
+            // Write metadata to cache
+            std::ofstream os(path, std::ios::binary);
+            std::ostream_iterator<char> osi(os);
+            std::copy(std::begin(*metadata), std::end(*metadata), osi);
+
+            return metadata;
         }
     }
 
-    auto buffer = std::make_shared<std::vector<char>>();
+    // Add trackers from magnet URL to the cached metadata
+    for (auto& tracker : atp.trackers) {
+        atp.ti->add_tracker(tracker);
+    }
 
-    // Bencode metadata into buffer
-    lt::bencode(
-        std::back_inserter(*buffer), lt::create_torrent(*atp.ti).generate());
+    auto entry = lt::create_torrent(*atp.ti).generate();
 
-    return buffer;
+    auto metadata = std::make_shared<std::vector<char>>();
+
+    // Bencode metadata into vector
+    lt::bencode(std::back_inserter(*metadata), entry);
+
+    return metadata;
 }
 
+// static
 std::shared_ptr<Download>
 Download::get_download(lt::add_torrent_params& atp, bool k)
 {
@@ -447,6 +468,7 @@ Download::get_download(lt::add_torrent_params& atp, bool k)
     return dl;
 }
 
+// static
 std::shared_ptr<Download>
 Download::get_download(char* md, size_t mdsz, std::string sp, bool k)
 {
@@ -473,6 +495,10 @@ Download::get_download(char* md, size_t mdsz, std::string sp, bool k)
 std::pair<int, uint64_t>
 Download::get_file(std::string path)
 {
+    D(printf("%s:%d: %s()\n", __FILE__, __LINE__, __func__));
+
+    download_metadata();
+
     const lt::file_storage& fs = m_th.torrent_file()->files();
     for (int i = 0; i < fs.num_files(); i++) {
         if (fs.file_path(i) == path)
@@ -485,44 +511,32 @@ Download::get_file(std::string path)
 std::string
 Download::get_name()
 {
+    D(printf("%s:%d: %s()\n", __FILE__, __LINE__, __func__));
+
+    download_metadata();
+
     return m_th.torrent_file()->name();
 }
 
 std::string
 Download::get_infohash()
 {
+    D(printf("%s:%d: %s()\n", __FILE__, __LINE__, __func__));
+
+    download_metadata();
+
     return lt::to_hex(m_th.torrent_file()->info_hash().to_string());
 }
 
 std::shared_ptr<std::vector<char>>
-Download::get_metadata()
+Download::get_metadata(MetadataProgressCb cb)
 {
     D(printf("%s:%d: %s()\n", __FILE__, __LINE__, __func__));
 
-    auto entry = lt::create_torrent(*m_th.torrent_file()).generate();
-
-    auto buffer = std::make_shared<std::vector<char>>();
-
-    // Bencode metadata into vector
-    lt::bencode(std::back_inserter(*buffer), entry);
-
-    return buffer;
-}
-
-std::shared_ptr<std::vector<char>>
-Download::get_metadata_and_write_to_file(std::string path)
-{
-    D(printf("%s:%d: %s()\n", __FILE__, __LINE__, __func__));
+    download_metadata(cb);
 
     auto entry = lt::create_torrent(*m_th.torrent_file()).generate();
 
-    std::filebuf fb;
-    fb.open(path, std::ios::out | std::ios::binary);
-    std::ostream os(&fb);
-
-    // Bencode metadata into file
-    lt::bencode(std::ostream_iterator<char>(os), entry);
-
     auto buffer = std::make_shared<std::vector<char>>();
 
     // Bencode metadata into vector
@@ -532,7 +546,7 @@ Download::get_metadata_and_write_to_file(std::string path)
 }
 
 void
-Download::download_metadata()
+Download::download_metadata(MetadataProgressCb cb)
 {
     D(printf("%s:%d: %s()\n", __FILE__, __LINE__, __func__));
 
@@ -545,19 +559,30 @@ Download::download_metadata()
 
     auto f = dlprom.get_future();
 
+    if (cb)
+        cb(0.0);
+
     // Wait for metadata to download
     while (!m_th.status().has_metadata) {
         auto r = f.wait_for(std::chrono::seconds(1));
         if (r == std::future_status::ready)
+            // At this point, we know either metadata download is done and we
+            // can return early, or there was error and get() will throw and
+            // exception.
             return f.get();
     }
+
+    if (cb)
+        cb(100.0);
 }
 
 void
-Download::download(lt::peer_request part)
+Download::download(lt::peer_request part, DataProgressCb cb)
 {
     D(printf("%s:%d: %s()\n", __FILE__, __LINE__, __func__));
 
+    download_metadata();
+
     if (m_th.have_piece(part.piece))
         return;
 
@@ -567,12 +592,20 @@ Download::download(lt::peer_request part)
 
     auto f = dlprom.get_future();
 
+    if (cb)
+        cb(0.0);
+
     // Wait for download
     while (!m_th.have_piece(part.piece)) {
         auto r = f.wait_for(std::chrono::seconds(1));
         if (r == std::future_status::ready)
+            // At this point, we know either download is done and we can return
+            // early, or there was error and get() will throw and exception.
             return f.get();
     }
+
+    if (cb)
+        cb(100.0);
 }
 
 ssize_t
@@ -580,6 +613,8 @@ Download::read(lt::peer_request part, char* buf, size_t buflen)
 {
     D(printf("%s:%d: %s()\n", __FILE__, __LINE__, __func__));
 
+    download_metadata();
+
     ReadPiecePromise rdprom(m_th.info_hash(), part.piece);
     AlertSubscriber<ReadPiecePromise> sub(m_session, &rdprom);
     vlc_interrupt_guard<ReadPiecePromise> intrguard(rdprom);
